#!/usr/bin/env python3
"""
Examples
--------
# Default option requires only the scaling factors and display name
$ xrandr-extend --dry-run vga
$ xrandr-extend vga
$ xrandr-extend hdmi

# Other options to extend the display
$ xrandr-extend --pan hdmi
$ xrandr-extend --only hdmi
$ xrandr-extend -e 1024 768 -n vga  # Pan with custom external resolution

"""
import argparse
from ast import literal_eval
from . import cmd, config


display_res_defaults = config.read()["resolutions"]


# Parse command-line arguments
parser = argparse.ArgumentParser(
    description="Extend a HIDPI screen to a normal DPI external display",
    epilog=__doc__,
    formatter_class=argparse.RawDescriptionHelpFormatter,
)
parser.add_argument(
    "profile",
    help="Use preset external resolution profiles (available: {}).".format(
        [k for k in display_res_defaults.keys() if k != "primary"]
    ),
)
parser.add_argument(
    "-p",
    "--pri-res",
    help="Modify preset resolution of primary display (default {})".format(
        display_res_defaults["primary"]
    ),
    nargs=2,
    type=int,
    default=literal_eval(display_res_defaults["primary"]),
)
parser.add_argument(
    "-e",
    "--ext-res",
    help="Modify preset resolution of ext. display (default based on profile)",
    nargs=2,
    type=int,
    default=None,
)
parser.add_argument(
    "-m", "--mirror", help="Mirror the ext. display", action="store_true"
)
parser.add_argument(
    "-n", "--pan", help="pan the position of ext. display", action="store_true"
)
parser.add_argument(
    "-o", "--only", help="extend and use only ext. display", action="store_true"
)
parser.add_argument(
    "-s",
    "--pos",
    help="set the position of ext. display explicitly",
    action="store_true",
)
parser.add_argument(
    "-d",
    "--dry-run",
    help="Preview command without executing it",
    action="store_true",
)


def run():
    args = parser.parse_args()

    display_names = cmd.display_names_from_providers()

    monitor1 = display_names["primary"]
    monitor2 = display_names[args.profile]

    # HIDPI monitor resolution
    A = args.pri_res[0]
    B = args.pri_res[1]

    # External non-HIDPI monitor resolution
    if args.ext_res is None:
        ext_res = display_res_defaults[args.profile]
        args.ext_res = literal_eval(ext_res)

    C = args.ext_res[0]
    D = args.ext_res[1]

    # Scaling factor
    E = round(A / C, 2)
    F = round(B / D, 2)
    E = F = max(E, F)

    # Prepare commands
    commands = ["xrandr --auto"]
    commands.extend(["xrandr --listmonitors"])
    if args.mirror:
        commands.extend(
            ["xrandr --output {} --scale {}x{}".format(monitor2, E, F)]
        )
    elif args.pan:
        commands.extend(
            [
                (
                    "xrandr --output {} --auto --output {} --auto --panning "
                    "{}x{}+{}+0 --scale {}x{} --right-of {}"
                ).format(
                    monitor1, monitor2, int(C * E), int(D * F), A, E, F, monitor1
                )
            ]
        )
    elif args.pos:
        commands.extend(
            [
                (
                    "xrandr --output {} --auto --pos 0x{}  --output {} "
                    "--scale {}x{} --auto --pos 0x0 --fb {}x{}"
                ).format(
                    monitor1,
                    int(D * F),
                    monitor2,
                    E,
                    F,
                    int(max(A, C * E)),
                    int(B + D * F),
                )
            ]
        )
    elif args.only:
        commands.extend(["xrandr --output {} --off".format(monitor1)])
    else:
        commands.extend(
            [
                (
                    "xrandr --output {} --auto --output {} --auto --scale {}x{} "
                    "--right-of {}"
                ).format(monitor1, monitor2, E, F, monitor1)
            ]
        )

    list(map(print, commands))
    if not args.dry_run:
        list(map(cmd.call, commands))


if __name__ == "__main__":
    run()



from pathlib import Path
from configparser import ConfigParser
import pkg_resources


# Default configuration file
DEFAULT_CFG_FILE = pkg_resources.resource_filename(__name__, "default.cfg")

# User-configuration file
CFG_FILE = Path.home() / ".config" / "xrandr-extend.cfg"


def default_cfg():
    config = ConfigParser()
    config.read(DEFAULT_CFG_FILE)

    return config


def read():
    if CFG_FILE.exists():
        config = default_cfg()
        config.read(CFG_FILE)
        return config
    else:
        return default_cfg()


def write_defaults():
    config = default_cfg()
    with open(CFG_FILE, "x") as f:
        config.write(f)


if __name__ == "__main__":
    print("Writing default configuration to {} ...".format(CFG_FILE))
    write_defaults()
    print("Done!")



"""Any function which executes a subprocess goes here."""

import subprocess
import sys
from shlex import split
from .config import read, CFG_FILE


def call(cmd):
    """Call a shell command

    Parameters
    ----------

    cmd : str

    """
    cmd = split(cmd)
    subprocess.call(cmd)


def display_names_from_providers():
    output = subprocess.check_output(split("xrandr --listproviders")).decode(
        "utf8"
    )
    # Configuration
    config = read()
    provider = output.lower().rstrip("\n").split(":")[-1]

    # Try to read a section such as:
    # [provider:modesetting]
    section = "provider:{}".format(provider)
    if config.has_section(section):
        display_names = config[section]
    else:
        print(
            (
                "Unknown X server provider. Output of `xrandr --listproviders` was:\n"
                + output
                + "Hint: perhaps you are using Wayland and not an X server? "
                + "If not simply add a section [{}] in {}".format(
                    section, CFG_FILE
                )
            )
        )
        sys.exit(1)

    return display_names



import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))


# Get the long description from the relevant file
with open(os.path.join(here, "README.md")) as f:
    long_description = f.read()

lines = long_description.splitlines(True)
long_description = "".join(lines[1:])

__version__ = "0.0.3"

console_scripts = [
    "xrandr-extend = xrandr_extend.main:run"
]

package_data={
    "xrandr_extend": ["default.cfg"]
}

setup(
    name="xrandr-extend",
    version=__version__,
    description=("Extend a HIDPI screen to a normal DPI external display"),
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="xrandr, hidpi, linux, external monitor",
    author="Ashwin Vishnu Mohanan",
    author_email="ashwinvis+gh@pm.me",
    url="https://github.org/ashwinvis/xrandr-extend",
    license="GPL",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Desktop Environment",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        # Specify the Python versions you support here. In particular,
        # ensure that you indicate whether you support Python 2,
        # Python 3 or both.
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    python_requires=">=3,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*",
    install_requires=["setuptools"],
    packages=find_packages(),
    package_data=package_data,
    entry_points={"console_scripts": console_scripts},
)
